#!/usr/bin/env python
import json
import re
import pandas as pd
from Bio import SeqIO
from abromics_galaxy_json_extractor.interfaces import (
    AbromicsResultIterator,
)


required_metadata = {
    "hid": "Historic ID for plasmidfinder file from galaxy",
    "analysis_software_version": "plasmidfinder version",
    "reference_database_version": "plasmidfinder DB version",
    "plasmid_result_tabular_path": "plasmidfinder results in tabular format",
    "plasmid_result_tabular_hid": "plasmidfinder results hid in Galaxy",
    "genome_hit_path": (
        "fasta file with hits in genome, doesn't work for multiple input"
    ),
    "genome_hit_hid": "Historic ID for genome hit file from galaxy",
    "plasmid_hit_path": (
        "fasta file with plasmid sequences, doesn't work for multiple input"
    ),
    "plasmid_hit_hid": "Historic ID for plasmid sequence hit file from galaxy",
}


class PlasmidFinderIterator(AbromicsResultIterator):
    """Plasmidfinder tool parser
    Filter results from Plasmidfinder json result
    Could also parse file for:
    Plasmid sequence hit file
    Genome sequence hit
    Args:
        AbromicsResultIterator (abc.ABCMeta): abstract class to the parser
    """

    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "plasmidfinder"
        self.metadata = metadata
        super().__init__(source, self.metadata)
        self.abromics_results["tool_has_database"] = True
        self.parse()

    def parse(self):
        """Parse plasmidfinder file"""
        if self.stream:
            self._get_plasmid_results("plasmid_result_tabular_path")
            self._get_genome_hit_results("genome_hit_path")
            self._get_genome_hit_results("plasmid_hit_path")

    def _get_plasmid_infos(self, plasmid_dict, plasmid_df):
        plasmid_info_df = pd.DataFrame.from_dict(plasmid_dict)
        plasmid_info_df["group"] = (
                plasmid_info_df["gene_name"]
                + "_"
                + plasmid_info_df["positions_in_contig"]
            )
        plasmid_info_df = plasmid_info_df.merge(
                plasmid_df, how="inner", on=["group"]
            )
        del plasmid_info_df['group']
        plasmid_info_df = plasmid_info_df.to_dict(orient="records")
        return(plasmid_info_df)

    def _get_mean_coverage(self):
        plasmid_df = pd.read_table(self.stream, header=0, sep="\t")
        plasmid_df["group"] = (
            plasmid_df["Note"] + "_" + plasmid_df["Position in contig"]
        )
        # Extract column names of input dataframe
        filtered_df = self._filter_columns(
            plasmid_df,
            colnames="coverage|group")
        return filtered_df

    def _filter_columns(self, df, colnames="coverage|group"):
        # Extract column names of input dataframe
        df_colnames = list(df.columns)
        # Regex pattern for re.search: searching all names in cols
        # regex_query = r"|".join(cols)
        regex_query = colnames
        # Search for each column name in df_colnames and keep matched only
        filtered_columns = list(
            filter(lambda v: re.search(regex_query, v), df_colnames)
        )
        df = df[filtered_columns]
        return df

    def __get_fasta_infos__(self, input: str):
        """Extract fasta informations
        Extract name, sequences, positions
        and IS information
        Returns:
            dict: sequence informations
        """
        fasta_sequences = SeqIO.parse(input, format="fasta")
        parsed_fasta_sequence = {}
        for seqname in fasta_sequences:
            sequence_infos = pd.DataFrame(seqname.description.split(" "))
            sequence_infos[[0, 1]] = sequence_infos[0].str.split(
                pat=":", n=1, expand=True
            )
            sequence_infos[[0, 1]] = sequence_infos[0].str.split(
                pat="=", n=1, expand=True
            )
            sequence_infos = sequence_infos.iloc[0:4, 0:2]
            sequence_infos = dict(
                zip(
                    sequence_infos[0],
                    sequence_infos[1],
                )
            )
            sequence_name = list(sequence_infos.keys())[0]
            sequence_infos.pop(sequence_name)
            parsed_fasta_sequence[sequence_name] = {
                "sequence_informations": sequence_infos,
                "nucleotide_sequence": seqname.seq.__str__(),
            }
        return parsed_fasta_sequence

    def _plasmid_infos(self):
        """Extract informations from plasmid json results
        Returns:
            list: a list of data
        """
        if self.stream:
            json_object = json.loads(self.stream.read())
            json_object = json_object[
                list(
                    json_object.keys())[0]
                ].pop("results")
            plasmid_family_list = list(json_object.keys())
            results = [
                self._extract_plasmid_family_infos(
                    json_input=json_object, plasmid_family=fam_name
                )
                for fam_name in plasmid_family_list
            ]
            clean_results = self._clean_plasmid_results(results)
        else:
            clean_results = []
        return clean_results

    def _clean_plasmid_results(self, raw_results):
        """Parse list of plasmid results

        Args:
            raw_results (_type_): a list of inputs

        Returns:
            list: list of infos to each plasmid hits
        """
        clean_results = []
        for i in range(len(raw_results)):
            for j in range(len(raw_results[i])):
                [
                    clean_results.append(k)
                    for k in raw_results[i][j]
                    if len(raw_results[i][j]) > 0
                ]
        return clean_results

    def _extract_plasmid_family_infos(self,
                                      json_input: dict,
                                      plasmid_family: str):
        """_summary_

        Args:
            json_input (dict): json input withvplasmid hits
            plasmid_family (str): Name of plasmid family
            type (gram - or enterobacteriaceae)

        Returns:
            list: plasmid list informations
        """
        plasmid_inc_list = list(json_input.get(plasmid_family).keys())
        return [
            self._extract_plasmid_incompatibility_infos(
                json_input=json_input,
                plasmid_group=plasmid_family,
                inc_name=inc
            )
            for inc in plasmid_inc_list
        ]

    def _extract_plasmid_incompatibility_infos(
        self, json_input: dict, plasmid_group: str, inc_name: str
    ):
        """Extract the plasmid incompatibility name to each hit

        Args:
            json_input (dict): json input file
            plasmid_group (str): group of plasmid family
            inc_name (str): inc name

        Raises:
            ValueError: json structure not good

        Returns:
            list: plasmid infos list
        """
        plasmid_hit = json_input.get(plasmid_group).get(inc_name)
        if isinstance(plasmid_hit, dict):
            return [
                self._extract_plasmid_infos(
                    contig_hit_name=key,
                    plasmid_hit_dict=plasmid_hit,
                    plasmid_group=plasmid_group,
                    inc_name=inc_name,
                )
                for key in plasmid_hit.keys()
            ]
        elif plasmid_hit == "No hit found":
            return []
        else:
            raise ValueError(
                f"Error when parsing plasmidfinder json results, \
                plasmid_hit is {plasmid_hit}"
                )

    def _extract_plasmid_infos(
        self,
        contig_hit_name,
        plasmid_hit_dict,
        plasmid_group: str,
        inc_name: str
    ):
        """Extract and translate plasmif infos

        Args:
            contig_hit_name (str): name of the hit
            plasmid_hit_dict (dict): dict of all hits
        """
        infos = plasmid_hit_dict.get(contig_hit_name)
        infos["inc_group"] = inc_name
        infos["ref_start"] = str.split(infos.get("position_in_ref"), "..")[0]
        infos["ref_end"] = str.split(infos.get("position_in_ref"), "..")[1]
        infos["contig_start"] = str.split(
            infos.get("positions_in_contig"), ".."
            )[0]
        infos["contig_end"] = str.split(
            infos.get("positions_in_contig"), ".."
            )[1]
        infos["contig_name"] = re.search(r"[^\s]+", contig_hit_name).group(0)
        infos["gene_name"] = infos.pop("note")
        infos["plasmid_group"] = plasmid_group
        return infos

    def _get_genome_hit_results(self, filetype):
        file_path = self.metadata.get(filetype)
        self._check_input_files(file_path, optional=True)
        if self.stream:
            reader = [self.__get_fasta_infos__(file_path)]
        else:
            reader = []

        self._fill_abromics_results(reader=reader, filetype=filetype)

    def _get_plasmid_results(self, filetype):
        plasmid_infos = self._plasmid_infos()
        file_path = self.metadata.get(filetype)
        self._check_input_files(file_path, optional=True)
        if self.stream:
            tmp_df = self._get_mean_coverage()
            plasmid_complete_df = self._get_plasmid_infos(
                plasmid_infos, tmp_df
                )
        else:
            plasmid_complete_df = plasmid_infos

        reader = []
        self._fill_abromics_results(reader=reader, filetype=filetype)
        self.abromics_results["results"][0]["content"] = plasmid_complete_df

    def _fill_abromics_results(self, reader, filetype: str):
        """Fill the final abromics object with results

        Args:
            filetype (str): _description_
            reader (pandas.core.frame.DataFrame): trimmed pandas dataframe
        """
        hid_info = filetype.replace("path", "hid")
        report_name = filetype.replace("_path", "")
        self.abromics_results["results"].append(
            {
                "name": report_name,
                "file_path": self.abromics_results.pop(
                    filetype),
                "hid": self.abromics_results.pop(
                    hid_info
                    ),
                "galaxy_id": "",
                "content": reader,
            }
        )
