#!/usr/bin/env python
import json
import re
import pandas as pd
import logging
from abromics_galaxy_json_extractor.interfaces import (
    AbromicsResultIterator,
)

required_metadata = {
    "hid": "Historic ID provided by Galaxy for resfinder file",
    "analysis_software_version": "tool version",
    "reference_database_version": "DB version",
    "mlst_file_path": "mlst output file from staramr",
    "mlst_file_hid": "Historic ID provided by Galaxy for mlst file",
    "plasmidfinder_file_path": "plasmid output file from staramr",
    "plasmidfinder_file_hid": "Historic ID provided by Galaxy for plasmid",
    "pointfinder_file_path": "pointfinder output file from staramr",
    "pointfinder_file_hid": "Historic ID provided by Galaxy for pointfinder",
    "setting_file_path": "setting file from staramr analysis",
    "setting_file_hid": "Historic ID provided by Galaxy for settings",
}


class StarAmrIterator(AbromicsResultIterator):
    """STARamr tool parser
    Filter results from resfinder file
    Could also parse results for:
    mlst scheme file
    plasmid file
    pointfinder file
    settings file
    Args:
        AbromicsResultIterator (abc.ABCMeta): abstract class to the parser
    """

    def __init__(self, source, metadata):
        metadata["analysis_software_name"] = "staramr"
        super().__init__(source, metadata)
        self.abromics_results["tool_has_database"] = False
        self.parse()

    def parse(self):
        """STARamr resfinder parser
        Parse resfinder result and could also
        parse other file:
        mlst, pointfinder, plasmidfinder and settings
        """
        # skip any manually specified fields for later
        if self.stream:
            reader = self._filter_dataframe_type(self.stream)
            self.abromics_results["results"][0]["content"] = reader
        {
            self._get_results(i)
            for i in self.metadata.keys()
            if 'file_path' in i
            }

    def _filter_dataframe_type(
        self,
        handle,
        file_type: str = None,
        header_position=0,
        separator="\t",
        orient_type="records",
    ):
        """Filter input dataframe
        Remove bad character and add coverage ratio value to
        resf-, plasmid- or pointfinder files
        Change locus structure to mlst results

        Args:
            handle (TextIOWrapper): input file
            header_position (int, optional): _description_. Defaults to 0.
            separator (str, optional): _description_. Defaults to "\t".
            orient_type (str, optional): _description_. Defaults to "records".

        Returns:
            dict: filtered data
        """
        try:
            reader = pd.read_table(
                handle,
                header=header_position,
                sep=separator,
            )
            if not reader.empty:
                reader = self._clean_df_characters(reader)
                if file_type == "finder":
                    reader["coverage_ratio"] = self._coverage_ratio_calculator(
                        reader
                        )
                elif file_type == "mlst":
                    reader = self._mlst_extract_infos(reader)
                try:
                    reader = json.loads(reader.to_json(orient=orient_type))
                    return reader
                except AttributeError:
                    reader = dict(zip(reader[0], reader[1]))
                    return reader
        except ValueError as ve:
            logging.error(f"{ve}, bad input file for {self.stream}")

    def _coverage_ratio_calculator(self, df):
        """calculate coverage ratio in the hit
        between query sequence and database match

        Args:
            df (pandas.core.frame.DataFrame): pandas dataframe

        Returns:
            pandas.core.frame.DataFrame: pandas dataframe with coverage ratio
        """
        coverage_ratio = df["HSP Length/Total Length"].str.split(
                pat="/", expand=True
                )
        coverage_ratio = pd.to_numeric(coverage_ratio[1]) / pd.to_numeric(
            coverage_ratio[0]
            )
        return coverage_ratio

    def _mlst_extract_infos(self, df):
        """Make a list of locus matchs

        Args:
            df (pandas.core.frame.DataFrame):
            pandas dataframe with mlst results
        Returns:
            pandas.core.frame.DataFrame: mlst results
        """
        colnames = list(df.columns)
        filtered_columns = list(
            filter(
                lambda v: re.search("Locus", v), colnames)
            )
        locus_list = df[filtered_columns].iloc[0].tolist()
        df = df.iloc[:, 0:3]
        df["Locus"] = ""
        df["Locus"] = df["Locus"].astype('object')
        df.at[0, "Locus"] = locus_list
        return df

    def _clean_df_characters(self, df):
        """Replace special character in pandas dataframe inputs
        Args:
            df (pandas.core.frame.DataFrame): pandas dataframe raw data

        Returns:
            pandas.core.frame.DataFrame: trimmed pandas dataframe
        """
        df = df.replace("^[ \t]+", "", regex=True)
        df = df.replace("[.]$", "", regex=True)
        df = df.replace(" ", "", regex=True)
        df.columns = df.columns.str.replace("%", "")
        return df

    def _get_results(self, filetype: str):
        """extract results to each dataframe and
        add output to th efinal abromics result list
        Args:
            filetype (str): type of input file
            need to filter dataframe
        """
        file_path = self.metadata.get(filetype)
        self._check_input_files(file_path, optional=True)
        if self.stream:
            reader = self._filter_dataframe_type(
                file_path,
                file_type=filetype
                )
        else:
            logging.info(f'No file associated to {filetype}')
            reader = []
        self._fill_abromics_results(reader=reader, filetype=filetype)

    def _fill_abromics_results(self, reader, filetype: str):
        """Fill the final abromics object with results

        Args:
            filetype (str): _description_
            reader (pandas.core.frame.DataFrame): trimmed pandas dataframe
        """
        hid_info = filetype.replace("path", "hid")
        report_name = filetype.replace("_path", "")
        try:
            self.abromics_results["results"].append(
                {
                    "name": report_name,
                    "file_path": self.abromics_results.pop(
                        filetype),
                    "hid": self.abromics_results.pop(
                        hid_info
                        ),
                    "galaxy_id": "",
                    "content": reader,
                }
            )
        except KeyError:
            logging.error(f'{KeyError}, {filetype} is not in item list')
            raise KeyError(f'{filetype} is not in item list')
